import { afterEach, beforeEach, describe, expect, test, vi } from 'vitest';
import { CMType, HMType } from '../../types';
import EmbeddedComponent from '../EmbeddedComponent';
const url = 'https://example.com';
const parentId = 'parent-container';
const value = { amountDecimal: 100 };
describe('EmbeddedComponent', () => {
    describe('creation', () => {
        let component;
        beforeEach(() => {
            component = new EmbeddedComponent(url, { amountDecimal: 100 });
        });
        test('sets value', () => {
            expect(component.value).toStrictEqual(value);
        });
        test('sets url', () => {
            expect(component.url).toBe(url);
        });
    });
    describe('iframe', () => {
        let document;
        let parentElement;
        let iframe;
        let window;
        let iframeWindow;
        beforeEach(() => {
            parentElement = {
                appendChild: vi.fn(),
            };
            iframe = {
                style: {},
                src: '',
                contentWindow: iframeWindow,
                setAttribute: vi.fn(),
                addEventListener: vi.fn(),
            };
            document = {
                createElement: vi.fn().mockReturnValue(iframe),
                querySelector: vi.fn().mockReturnValue(parentElement),
            };
            window = {
                addEventListener: vi.fn(),
            };
            vi.stubGlobal('document', document);
            vi.stubGlobal('window', window);
        });
        afterEach(() => {
            vi.unstubAllGlobals();
        });
        test('mount', () => {
            const component = new EmbeddedComponent(url, value);
            component.mount(parentId);
            expect(iframe?.src).toBe(url);
            expect(parentElement.appendChild).toHaveBeenCalledWith(iframe);
        });
        test('on', () => {
            const component = new EmbeddedComponent(url, value);
            const messageType = 'customEvent';
            const callback = vi.fn();
            component.on(messageType, callback);
            component.mount(parentId);
            // Simulate receiving a message of the specified type
            // and assert that the callback is called with the correct payload.
        });
        test('off', () => {
            const component = new EmbeddedComponent(url, value);
            const messageType = 'customEvent';
            const callback = vi.fn();
            component.on(messageType, callback);
            // Simulate receiving a message of the specified type
            // and assert that the callback is called with the correct payload.
            component.off(messageType, callback);
            // Simulate receiving another message of the specified type
            // and assert that the callback is not called.
        });
        describe('connected to iframe', () => {
            let messageHandler;
            let port;
            let handshake;
            let component;
            beforeEach(() => {
                port = {
                    postMessage: vi.fn(),
                    start: vi.fn(),
                    onmessage: null,
                };
                handshake = {
                    source: iframe.contentWindow,
                    ports: [port],
                };
                window.addEventListener = vi
                    .fn()
                    .mockImplementation((event, handler) => {
                    messageHandler = handler;
                });
                component = new EmbeddedComponent(url, value);
                component.mount(parentId);
                expect(messageHandler).toBeDefined();
                messageHandler(handshake);
            });
            test('send', () => {
                const message = {
                    type: HMType.UPDATE,
                    value: { amountDecimal: 100 },
                };
                component.send(message);
                // Assert that the message is sent to the embedded component
                // and handled correctly.
            });
            test('update', () => {
                const newValue = { amountDecimal: 200 };
                component.update(newValue);
                expect(component.value).toBe(newValue);
            });
            // TODO not sure this actually does what I want it to do
            test('completion', async () => {
                const response = { valid: true, value: 'completed' };
                expect(port.onmessage).toBeDefined();
                const promise = component.completion();
                // @ts-expect-error - we've already verified that port exists
                port.onmessage({
                    data: { type: CMType.COMPLETE, response },
                });
                promise.then((result) => {
                    expect(result).toBe(response);
                });
                return promise;
                // Assert that the completion promise resolves with the expected response.
            });
        });
    });
});
